# -*- eval: (blacken-mode) -*-
import logging
import pytest
import numpy as np
from numpy.testing import assert_allclose

import usadelndsoc
import usadelndsoc.solver

usadelndsoc.logger.setLevel(logging.DEBUG)


def basic_setup(terminals=False, nx=15, ny=5):
    Delta = np.zeros((4, 4))
    Delta[:2, 2:] = np.eye(2)
    Delta[2:, :2] = np.eye(2)

    s = usadelndsoc.solver.Solver(nx=nx, ny=ny)
    s.Omega[...] = 1j * Delta
    s.alpha = 0.0
    s.Lx = 10
    s.Ly = 10

    if terminals:
        s.mask[0, :] = usadelndsoc.solver.MASK_TERMINAL
        s.mask[-1, :] = usadelndsoc.solver.MASK_TERMINAL

    return s


# s.solve(omega=150, maxiter=300, preconditioner="none")


def test_solve_dos():
    s = basic_setup()
    E = np.linspace(-3, 3, 101) + 0.05j
    res = s.solve_many(omega=-1j * E)
    g = res.G[:, 10, 2, 0, 0]
    g_an = -1j * E / np.sqrt(1 - E**2)
    assert_allclose(g, g_an, rtol=1e-4)


@pytest.mark.parametrize("omega", [0.5 + 0.8j, -0.5 + 0.8j])
def test_result_S_J0(omega):
    s = basic_setup()
    s.Delta[...] = np.eye(2)
    res = s.solve_many(omega=omega)
    S = res.S

    vol = s.Lx * s.Ly / (s.shape[0] * s.shape[1]) * 4j

    assert_allclose(S[:, :, :2, :2], vol * res.G)
    assert_allclose(S[:, :, :2, 2:], vol * res.F)
    assert_allclose(S[:, :, 2:, :2], vol * res.Fc)
    assert_allclose(S[:, :, 2:, 2:], vol * res.Gc)

    # Check vs. analytic
    Gan = omega / np.sqrt(omega**2 + 1) * np.eye(2)
    Fan = 1 / np.sqrt(omega**2 + 1) * np.eye(2)
    Fcan = 1 / np.sqrt(omega**2 + 1) * np.eye(2)
    Gcan = -omega / np.sqrt(omega**2 + 1) * np.eye(2)

    shp = res.G.shape

    assert_allclose(res.G, np.broadcast_to(Gan, shp))
    assert_allclose(res.F, np.broadcast_to(Fan, shp))
    assert_allclose(res.Fc, np.broadcast_to(Fcan, shp))
    assert_allclose(res.Gc, np.broadcast_to(Gcan, shp))

    # There are no gradients, hence no currents either
    J = res.J
    assert_allclose(J, 0, atol=1e-5)
